"Cognite API SDK\n\nThis package has been created to ensure excellent excellent user experience for data scientists using the Cognite\nData Platform (CDP).\n"
__all__ = ["v04", "v05", "v06", "preprocessing", "config", "data_transfer_service"]
__version__ = "0.11.15"
